/*  Copyright (c) MIPoPS. All Rights Reserved.
 *
 *  Use of this source code is governed by a Unlicense license that can
 *  be found in the LICENSE file in the root of the source tree.
 */

//---------------------------------------------------------------------------
#include "Common/Core.h"
#include "ZenLib/Ztring.h" // TODO: remove that
using namespace ZenLib;
//---------------------------------------------------------------------------

//***************************************************************************
// Constructor/Destructor
//***************************************************************************

void __stdcall Event_CallBackFunction(unsigned char* Data_Content, size_t Data_Size, void* UserHandler_Void)
{
    //Retrieving UserHandler
    Core*                               UserHandler=(Core*)UserHandler_Void;
    struct MediaInfo_Event_Generic*     Event_Generic=(struct MediaInfo_Event_Generic*) Data_Content;
    unsigned char                       ParserID;
    unsigned short                      EventID;
    unsigned char                       EventVersion;

    //Integrity test
    if (Data_Size<4)
        return; //There is a problem

    //Retrieving EventID
    ParserID    =(unsigned char) ((Event_Generic->EventCode&0xFF000000)>>24);
    EventID     =(unsigned short)((Event_Generic->EventCode&0x00FFFF00)>>8 );
    EventVersion=(unsigned char) ( Event_Generic->EventCode&0x000000FF     );
    switch (ParserID)
    {
        case MediaInfo_Parser_None :    
                switch (EventID)
                {    
                    case MediaInfo_Event_General_Start                                          : if (EventVersion==0 && Data_Size==sizeof(struct MediaInfo_Event_General_Start_0)) UserHandler->Parse_NewFile(); break;
                    default                                                                     : ;
                }
                break;
        case MediaInfo_Parser_DvDif :    
                switch (EventID)
                {    
                    case MediaInfo_Event_DvDif_Analysis_Frame                                   : if (EventVersion==0 && Data_Size==sizeof(struct MediaInfo_Event_DvDif_Analysis_Frame_0)) UserHandler->Parse_AddFrame((MediaInfo_Event_DvDif_Analysis_Frame_0*)Event_Generic); break;
                    default                                                                     : ;
                }
                break;
        default : ; //ParserID is unknown
    }
}

//***************************************************************************
// Constructor/Destructor
//***************************************************************************

//---------------------------------------------------------------------------
Core::Core()
{
    MI=new MediaInfoNameSpace::MediaInfoList;
    MI->Option(__T("File_Event_CallBackFunction"), __T("CallBack=memory://")+Ztring::ToZtring((size_t)&Event_CallBackFunction)+__T(";UserHandler=memory://")+Ztring::ToZtring((size_t)this));
    MI->Option(__T("ParseSpeed"), __T("1.000"));
    MI->Option(__T("File_DvDif_Analysis"), __T("1"));
    Xml=false;
}

Core::~Core()
{
    delete MI;

    Parse_Clear();
}

//***************************************************************************
// Process
//***************************************************************************

//---------------------------------------------------------------------------
void Core::Process()
{
    Parse_Clear();
    
    MI->Close();
    for (auto Input : Inputs)
        MI->Open(Input);
}

//---------------------------------------------------------------------------
float Core::State ()
{
    return (((float)MI->State_Get())/10000);
}

//***************************************************************************
// Parse
//***************************************************************************

//---------------------------------------------------------------------------
void Core::Parse_NewFile()
{
    ParsedFrames.push_back(new std::vector<MediaInfo_Event_DvDif_Analysis_Frame_0>);
}

//---------------------------------------------------------------------------
void Core::Parse_AddFrame(const MediaInfo_Event_DvDif_Analysis_Frame_0* FrameData)
{
    MediaInfo_Event_DvDif_Analysis_Frame_0 ToPush;
    std::memcpy(&ToPush, FrameData, sizeof(MediaInfo_Event_DvDif_Analysis_Frame_0));
    if (ToPush.Errors)
    {
        size_t SizeToCopy = std::strlen(FrameData->Errors) + 1;
        ToPush.Errors = new char[SizeToCopy];
        std::memcpy(ToPush.Errors, FrameData->Errors, SizeToCopy);
    }
    ParsedFrames[ParsedFrames.size() - 1]->push_back(ToPush);
}

//---------------------------------------------------------------------------
void Core::Parse_Clear()
{
    for (size_t File_Pos = 0; File_Pos < ParsedFrames.size(); File_Pos++)
    {
        for (size_t Pos = 0; Pos < ParsedFrames[File_Pos]->size(); Pos++)
            if (ParsedFrames[File_Pos]->at(Pos).Errors)
                delete[] ParsedFrames[File_Pos]->at(Pos).Errors;
        delete ParsedFrames[File_Pos];
    }
    ParsedFrames.clear();
}


//---------------------------------------------------------------------------
string Core::OutputXml()
{
    string Text;

    // XML header
    Text += "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
    Text += "<dvrescue xmlns=\"https://mediaarea.net/dvrescue\" version=\"1.0\">\n";

    auto File_Count = MI->Count_Get();
    for (auto File_Pos = 0; File_Pos < File_Count; File_Pos++)
    {
        // Media header
        Text += "\t<media ref=\"" + Ztring(MI->Get(File_Pos, Stream_General, 0, __T("CompleteName"))).To_UTF8() + "\">\n";

        // Media footer
        Text += "\t</media>\n";
    }

    // XML footer
    Text += "</dvrescue>\n";

    return Text;
}
